@@ -1,5 +1,5 @@
 import support
-from support import TODO, TestCase
+from support import TODO, TestCase, test_hash, test_equality
 
 if __name__ == '__main__':
 	support.adjust_path()
@@ -53,15 +53,36 @@ class SingleTests(TestCase):
 	def test_equality(self):
 		from typecheck import Single
 		
-		int_1 = Single(int)
-		int_2 = Single(int)
-		flt_1 = Single(float)
-		flt_2 = Single(float)
+		class A(object): pass
+		class B(A): pass
 		
-		assert int_1 == int_2
-		assert not int_1 != int_2
-		assert int_1 != flt_1
-		assert not int_1 == flt_1
+		eq_tests = [
+			(Single(int), Single(int)),
+			(Single(A), Single(A)),
+			(Single(B), Single(B)) ]
+			
+		ne_tests = [
+			(Single(int), Single(float)),
+			(Single(A), Single(B)) ]
+		
+		test_equality(eq_tests, ne_tests)
+		
+	def test_hash(self):
+		from typecheck import Single
+		
+		class A(object): pass
+		class B(A): pass
+		
+		eq_tests = [
+			(Single(int), Single(int)),
+			(Single(A), Single(A)),
+			(Single(B), Single(B)) ]
+			
+		ne_tests = [
+			(Single(int), Single(float)),
+			(Single(A), Single(B)) ]
+		
+		test_hash(eq_tests, ne_tests)
 
 class DictTests(TestCase):
 	def setUp(self):
@@ -115,25 +136,40 @@ class DictTests(TestCase):
 		else:
 			self.fail("Passed incorrectly")
 			
-	def test_eq_ne_operators(self):
-		from typecheck import Dict, Tuple, List
+	def test_equality(self):
+		from typecheck import Dict
 		
-		dict1 = Dict(str, int)
-		dict2 = Dict(int, int)
-		dict3 = Dict(str, int)
-		tup1 = Tuple(str, int)
-		list1 = List(str, int)
-
-		assert dict1 == dict3
-		assert dict1 != dict2
-		assert dict1 != tup1
-		assert tup1 != dict1
-		assert dict1 != list1
-		assert list1 != dict1
-		assert dict2 != tup1
-		assert dict2 != list1
-		assert tup1 != dict2
-		assert list1 != dict2
+		class A(object): pass
+		class B(A): pass
+		
+		eq_tests = [
+			(Dict(str, int), Dict(str, int)),
+			(Dict(str, A), Dict(str, A)),
+			(Dict(str, Dict(str, int)), Dict(str, Dict(str, int))) ]
+			
+		ne_tests = [
+			(Dict(str, int), Dict(int, str)),
+			(Dict(str, int), {'a': 5}),
+			(Dict(str, Dict(str, int)), Dict(str, Dict(int, str))) ]
+		
+		test_equality(eq_tests, ne_tests)
+		
+	def test_hash(self):
+		from typecheck import Dict
+		
+		class A(object): pass
+		class B(A): pass
+		
+		eq_tests = [
+			(Dict(str, int), Dict(str, int)),
+			(Dict(str, A), Dict(str, A)),
+			(Dict(str, Dict(str, int)), Dict(str, Dict(str, int))) ]
+			
+		ne_tests = [
+			(Dict(str, int), Dict(int, str)),
+			(Dict(str, Dict(str, int)), Dict(str, Dict(int, str))) ]
+		
+		test_hash(eq_tests, ne_tests)
 			
 class TupleTests(TestCase):
 	def setUp(self):
@@ -182,22 +218,46 @@ class TupleTests(TestCase):
 		else:
 			self.fail("Passed incorrectly")
 			
-	def test_eq_ne_operators(self):
-		from typecheck import List, Tuple
-	
-		tup1 = Tuple(int, float, int)
-		tup2 = Tuple(float, int, float)
-		tup3 = Tuple(int, float, int)
-		tup4 = Tuple(float)
-		list1 = List(int, float, int)
+	def test_equality(self):
+		from typecheck import Tuple
 		
-		assert tup1 != tup2
-		assert tup1 == tup3
-		assert tup1 != tup4
-		assert tup1 != list1
-		assert tup2 != tup3
-		assert tup2 != tup4
-		assert list1 != tup1
+		class A(object): pass
+		class B(A): pass
+		
+		eq_tests = [
+			(Tuple(str, int), Tuple(str, int)),
+			(Tuple(str, A), Tuple(str, A)),
+			(Tuple(), Tuple()),
+			(Tuple(str, Tuple(str, int)), Tuple(str, Tuple(str, int))) ]
+			
+		ne_tests = [
+			(Tuple(str, int), Tuple(int, str)),
+			(Tuple(str, int), (str, int)),
+			(Tuple(A, A), Tuple(A, B)),
+			(Tuple(str, int, float), Tuple()),
+			(Tuple(str, Tuple(str, int)), Tuple(str, Tuple(int, str))) ]
+		
+		test_equality(eq_tests, ne_tests)
+		
+	def test_hash(self):
+		from typecheck import Tuple
+		
+		class A(object): pass
+		class B(A): pass
+		
+		eq_tests = [
+			(Tuple(str, int), Tuple(str, int)),
+			(Tuple(str, A), Tuple(str, A)),
+			(Tuple(), Tuple()),
+			(Tuple(str, Tuple(str, int)), Tuple(str, Tuple(str, int))) ]
+			
+		ne_tests = [
+			(Tuple(str, int), Tuple(int, str)),
+			(Tuple(A, A), Tuple(A, B)),
+			(Tuple(str, int, float), Tuple()),
+			(Tuple(str, Tuple(str, int)), Tuple(str, Tuple(int, str))) ]
+		
+		test_hash(eq_tests, ne_tests)
 		
 	def test_empty_tuple_success(self):
 		from typecheck import Tuple
@@ -252,19 +312,48 @@ class SingleType_ListTests(TestCase):
 		else:
 			self.fail("Passed incorrectly")
 			
-	def test_eq_ne_operators(self):
-		from typecheck import List, Tuple
-	
-		list1 = List(int)
-		list2 = List(float)
-		list3 = List(int)
-		tup1 = Tuple(int)
+	def test_equality(self):
+		from typecheck import List
 		
-		assert list1 == list3
-		assert list2 != list3
-		assert list1 != list2
-		assert list1 != tup1
-		assert tup1 != list1
+		class A(object): pass
+		class B(A): pass
+		
+		eq_tests = [
+			(List(str), List(str)),
+			(List(A), List(A)),
+			(List(), List()),
+			(List(List(int)), List(List(int))) ]
+			
+		ne_tests = [
+			(List(str), List(int)),
+			(List(A), List(B)),
+			(List(), List(int)),
+			(List(List(int)), List(List(List(int)))),
+			(List(int), List(int, int)),
+			(List(int), [int]) ]
+		
+		test_equality(eq_tests, ne_tests)
+		
+	def test_hash(self):
+		from typecheck import List
+		
+		class A(object): pass
+		class B(A): pass
+		
+		eq_tests = [
+			(List(str), List(str)),
+			(List(A), List(A)),
+			(List(), List()),
+			(List(List(int)), List(List(int))) ]
+			
+		ne_tests = [
+			(List(str), List(int)),
+			(List(A), List(B)),
+			(List(), List(int)),
+			(List(List(int)), List(List(List(int)))),
+			(List(int), List(int, int)) ]
+		
+		test_hash(eq_tests, ne_tests)
 			
 class Pattern_ListTests(TestCase):
 	def setUp(self):
@@ -316,22 +405,48 @@ class Pattern_ListTests(TestCase):
 		else:
 			self.fail("Passed incorrectly")
 			
-	def test_eq_ne_operators(self):
-		from typecheck import List, Tuple
-	
-		list1 = List(int, float)
-		list2 = List(float, int)
-		list3 = List(int)
-		list4 = List(int, float)
-		tup1 = Tuple(int, float)
+	def test_equality(self):
+		from typecheck import List
 		
-		assert list1 != list2
-		assert list1 != list3
-		assert list1 == list4
-		assert list2 != list3
-		assert list2 != list4
-		assert list1 != tup1
-		assert tup1 != list1
+		class A(object): pass
+		class B(A): pass
+		
+		eq_tests = [
+			(List(str, str), List(str, str)),
+			(List(A, B), List(A, B)),
+			(List(List(int, int), int), List(List(int, int), int)) ]
+			
+		ne_tests = [
+			(List(str, int), List(int, str)),
+			(List(A, B), List(B, B)),
+			(List(A, B), List(A, A)),
+			(List(), List(int, int)),
+			(List(List(int, int)), List(List(List(int, int)))),
+			(List(int, int), List(int, int, int)),
+			(List(int, int), [int, int]) ]
+		
+		test_equality(eq_tests, ne_tests)
+		
+	def test_hash(self):
+		from typecheck import List
+		
+		class A(object): pass
+		class B(A): pass
+		
+		eq_tests = [
+			(List(str, str), List(str, str)),
+			(List(A, B), List(A, B)),
+			(List(List(int, int), int), List(List(int, int), int)) ]
+			
+		ne_tests = [
+			(List(str, int), List(int, str)),
+			(List(A, B), List(B, B)),
+			(List(A, B), List(A, A)),
+			(List(), List(int, int)),
+			(List(List(int, int)), List(List(List(int, int)))),
+			(List(int, int), List(int, int, int)) ]
+		
+		test_hash(eq_tests, ne_tests)
 			
 class NestedTests(TestCase):
 	def test_patterned_lists_in_lists(self):
