@@ -62,6 +62,9 @@ class _TC_IndexError(_TC_NestedError):
 	def message(self):
 		return ("at index %d" % self.index) + _TC_NestedError.message(self)
 
+# _TC_DictError exists as a wrapper around dict-related exceptions.
+# It provides a single place to sort the bad dictionary's keys in the error
+# message.
 class _TC_DictError(_TC_NestedError):
 	def format_bad_object(self, bad_object):
 		message = "in {%s}, " % ', '.join(["%s: %s" % (repr(k), repr(bad_object[k])) for k in sorted(bad_object.keys())])
@@ -104,7 +107,8 @@ class _TC_GeneratorError(_TC_NestedError):
 		bad_obj, start_message = self.inner.format_bad_object(bad_object)
 		message = "At yield #%d: %s" % (self.yield_no, bad_obj)
 		return (message, start_message)
-		
+
+### These next three exceptions exist to give HasAttr better error messages		
 class _TC_AttrException(_TC_Exception):
 	def __init__(self, attr):
 		_TC_Exception.__init__(self, attr)
@@ -123,11 +127,49 @@ class _TC_MissingAttrError(_TC_AttrException):
 	def message(self):
 		return "missing attribute %s" % self.attr
 
+# This is like _TC_LengthError for YieldSeq		
+class _TC_YieldCountError(_TC_Exception):
+	def __init__(self, expected):
+		_TC_Exception.__init__(self, expected)
+		
+		self.expected = expected
+		
+	def format_bad_object(self, bad_object):
+		return ("", self)
+		
+	def message(self):
+		plural = "s"
+		if self.expected == 1:
+			plural = ""
+		
+		return "only expected the generator to yield %d time%s" % (self.expected, plural)
+
+# This exists to provide more detailed error messages about why a given
+# Xor() assertion failed
+class _TC_XorError(_TC_NestedError):
+	def __init__(self, matched_conds, inner_exception):
+		assert matched_conds in (0, 2)
+		assert isinstance(inner_exception, _TC_TypeError)
+		
+		_TC_Exception.__init__(self, matched_conds, inner_exception)
+		_TC_NestedError.__init__(self, inner_exception)
+		self.matched_conds = matched_conds
+		
+	def message(self):
+		if self.matched_conds == 0:
+			m = "neither assertion"
+		else:
+			m = "both assertions"
+	
+		return _TC_NestedError.message(self) + " (matched %s)" % m
+
 ### The following exist to provide detailed TypeSignatureErrors
 class _TS_Exception(Exception):
 	def message(self):
 		raise NotImplementedError("Incomplete _TS_Exception subclass (%s)" % str(self.__class__))
 
+# This is used when there was an error related to an auto-unpacked tuple
+# in the function's signature
 class _TS_TupleError(_TS_Exception):
 	def __init__(self, parameters, types):
 		parameters = _tuplize(parameters)
@@ -166,7 +208,9 @@ class _TS_MissingTypeError(_TS_Exception):
 		
 	def message(self):
 		return "parameter '%s' lacks a type" % self.parameter
-		
+
+# If the user has given a keyword parameter a type both positionally and
+# with a keyword argument, this will be raised		
 class _TS_TwiceTypedError(_TS_Exception):
 	def __init__(self, parameter, kw_type, pos_type):
 		_TS_Exception.__init__(self, parameter, kw_type, pos_type)
@@ -308,14 +352,13 @@ def calculate_type(obj):
 
 ### The following classes are the work-horses of the typechecker
 
-### The base class for all the other utility classes
+# The base class for all the other utility classes
 class CheckType(object):
-	def __str__(self):
-		return self.__repr__()
-
 	def __repr__(self):
 		return type(self).name + '(' + ', '.join(sorted(repr(t) for t in self._types)) + ')'
 
+	__str__ = __repr__
+
 	def __eq__(self, other):
 		return not self != other
 		
@@ -323,10 +366,10 @@ class CheckType(object):
 		return not self == other
 
 	def __hash__(self):
-		raise NotImplementedError("Incomplete CheckType subclass")
+		raise NotImplementedError("Incomplete CheckType subclass: %s" % self.__class__)
 	
 	def __typecheck__(self, func, obj):
-		raise NotImplementedError("Incomplete CheckType subclass")
+		raise NotImplementedError("Incomplete CheckType subclass: %s" % self.__class__)
 	
 	@classmethod
 	def __typesig__(cls, obj):
@@ -337,11 +380,7 @@ class Single(CheckType):
 	name = "Single"
 
 	def __init__(self, type):
-		# XXX: Is this first test correct?
-		# Shouldn't it be isinstance(type, Single)?
-		if isinstance(type, CheckType):
-			self.type = type.type
-		elif not isinstance(type, (types.ClassType, types.TypeType)):
+		if not isinstance(type, (types.ClassType, types.TypeType)):
 			raise TypeError("Cannot type-check a %s" % type(type))
 		else:
 			self.type = type
@@ -359,7 +398,9 @@ class Single(CheckType):
 		
 	def __hash__(self):
 		return hash(str(hash(self.__class__)) + str(hash(self.type)))
-		
+	
+	# XXX Is this really a good idea?
+	# Removing this only breaks 3 tests; that seems suspiciously low
 	def __repr__(self):
 		return repr(self.type)
 		
@@ -368,15 +409,13 @@ class Single(CheckType):
 		if isinstance(obj, (types.ClassType, types.TypeType)):
 			return Single(obj)
 
-### Provide a way to enforce the empty-ness of lists and dicts
-### That is, Empty(list) matches only []'s, and Empty(dict)
-### only approves {}'s		
+### Provide a way to enforce the empty-ness of iterators	
 class Empty(Single):
 	name = "Empty"
 	
 	def __init__(self, type):
-		if type not in (dict, list):
-			raise TypeError("Can only assert emptyness for dicts and lists")
+		if not hasattr(type, '__len__'):
+			raise TypeError("Can only assert emptyness for types with __len__ methods")
 		
 		Single.__init__(self, type)
 
@@ -396,7 +435,7 @@ class Dict(CheckType):
 		self.__check_key = Type(key)
 		self.__check_val = Type(val)
 		
-		self.type = { key: val }
+		self.type = {key: val}
 		self._types = [key, val]
 		
 	def __typecheck__(self, func, to_check):
@@ -422,8 +461,15 @@ class Dict(CheckType):
 		return self.type == other.type
 		
 	def __hash__(self):
-		return hash(str(hash(self.__class__)) + str(hash(self.type)))
+		cls = self.__class__
+		key = self.__check_key
+		val = self.__check_val
 		
+		def strhash(obj):
+			return str(hash(obj))
+	
+		return hash(''.join(map(strhash, [cls, key, val])))
+	
 	@classmethod
 	def __typesig__(cls, obj):
 		if isinstance(obj, dict):
@@ -478,7 +524,10 @@ class List(CheckType):
 		return True
 		
 	def __hash__(self):
-		return hash(str(hash(self.__class__)) + str([hash(t) for t in self.type]))
+		def strhash(obj):
+			return str(hash(obj))
+			
+		return hash(''.join(map(strhash, [self.__class__] + self._types)))
 	
 	@classmethod	
 	def __typesig__(cls, obj):
@@ -534,6 +583,14 @@ class TypeVariables(CheckType):
 		return "TypeVariable(%s)" % self.type
 		
 	__repr__ = __str__
+	
+	def __hash__(self):
+		return hash(str(self.__class__) + str(hash(self.type)))
+		
+	def __eq__(self, other):
+		if self.__class__ is not other.__class__:
+			return False
+		return self.type == other.type
 		
 	def __typecheck__(self, func, to_check):
 		name = self.type
@@ -622,18 +679,28 @@ class Any(CheckType):
 
 ### Base class for Or() and And()
 class _Boolean(CheckType):
-	def __init__(self, type, *types):
-		self._types = set([Type(t) for t in (type,)+types])
+	def __init__(self, first_type, second_type, *types):
+		self._types = set()
+		
+		for t in (first_type, second_type)+types:
+			if type(t) is type(self):
+				self._types.update(t._types)
+			else:
+				self._types.add(Type(t))
+				
+		if len(self._types) < 2:
+			raise TypeError("there must be at least 2 distinct parameters to __init__()")
+
 		self.type = self
 		
 	def __eq__(self, other):
 		if other.__class__ is not self.__class__:
 			return False
 
-		return self._types.issubset(other._types)
+		return self._types == other._types
 		
 	def __hash__(self):
-		return hash(str(self.__class__) + str([hash(t) for t in self._types]))
+		return hash(str(hash(self.__class__)) + str(hash(frozenset(self._types))))
 
 class Or(_Boolean):
 	name = "Or"
@@ -661,6 +728,13 @@ class And(_Boolean):
 class Not(Or):
 	name = "Not"
 	
+	# We override _Boolean's __init__ so that we can accept a single
+	# condition
+	def __init__(self, first_type, *types):
+		self._types = set([Type(t) for t in (first_type,)+types])
+						
+		self.type = self
+	
 	def __typecheck__(self, func, to_check):
 		# Or does our work for us, but we invert its result
 		try:
@@ -669,6 +743,25 @@ class Not(Or):
 			return
 		raise _TC_TypeError(to_check, self)
 		
+class Xor(_Boolean):
+	name = "Xor"
+
+	def __typecheck__(self, func, to_check):
+		already_met_1_cond = False
+		
+		for typ in self._types:
+			try:
+				check_type(typ, func, to_check)
+			except _TC_Exception:
+				pass
+			else:
+				if already_met_1_cond:
+					raise _TC_XorError(2, _TC_TypeError(to_check, self))
+				already_met_1_cond = True
+					
+		if not already_met_1_cond:
+			raise _TC_XorError(0, _TC_TypeError(to_check, self))		
+		
 class IsCallable(CheckType):
 	def __init__(self):
 		self.type = self
@@ -678,6 +771,14 @@ class IsCallable(CheckType):
 		
 	__repr__ = __str__
 	
+	# They're all the same
+	# XXX Change IsCallable to a singleton class	
+	def __hash__(self):
+		return id(self.__class__)
+	
+	def __eq__(self, other):
+		return self.__class__ is other.__class__
+	
 	def __typecheck__(self, func, to_check):
 		if not callable(to_check):
 			raise _TC_TypeError(to_check, 'a callable')
@@ -708,13 +809,105 @@ class HasAttr(CheckType):
 			except _TC_Exception, e:
 				raise _TC_AttrError(attr, e)
 				
+	def __eq__(self, other):
+		if self.__class__ is not other.__class__:
+			return False
+		return self._attr_types == other._attr_types
+		
+	def __hash__(self):
+		return hash(str(hash(self.__class__)) + str(hash(str(self._attr_types))))
+		
+	def __str__(self):
+		any_type = []
+		spec_type = {}
+		
+		any = Any()
+		
+		for (attr, typ) in self._attr_types.items():
+			if typ == any:
+				any_type.append(attr)
+			else:
+				spec_type[attr] = typ
+
+		msg = [t for t in (any_type, spec_type) if len(t)]				
+	
+		return "HasAttr(" + ', '.join(map(str, msg)) + ")"
+		
+	__repr__ = __str__
+				
 class IsIterable(CheckType):
 	def __init__(self):
 		self.type = self
 		
+	def __eq__(self, other):
+		return self.__class__ is other.__class__
+	
+	# They're all the same
+	# XXX Change IsIterable to a singleton class	
+	def __hash__(self):
+		return id(self.__class__)
+		
+	def __str__(self):
+		return "IsIterable()"
+		
+	__repr__ = __str__
+		
 	def __typecheck__(self, func, to_check):
 		if not (hasattr(to_check, '__iter__') and callable(to_check.__iter__)):
 			raise _TC_TypeError(to_check, "an iterable")
+			
+class YieldSeq(CheckType):
+	_index_map = {}
+
+	def __init__(self, type_1, type_2, *types):
+		self.type = self
+		
+		self._type = [type_1, type_2] + list(types)
+		self._types = [Type(t) for t in self._type]
+		
+	def __hash__(self):
+		return id(self)
+		
+	def __str__(self):
+		return "YieldSeq(" + ", ".join(map(str, self._type)) + ")"
+		
+	__repr__ = __str__
+	
+	def __eq__(self, other):
+		if self.__class__ is not other.__class__:
+			return False
+		return self._types == other._types
+		
+	def __hash__(self):
+		return hash(str(self.__class__) + str([hash(t) for t in self._types]))
+	
+	# We have to use __{start,stop}checking__ so that the indexes get
+	# reset every time we run through the typechecking sequence
+	@classmethod
+	def __startchecking__(cls, gen):
+		if isinstance(gen, GeneratorType):
+			cls._index_map[gen] = {}
+	
+	@classmethod	
+	def __stopchecking__(cls, gen):
+		if gen in cls._index_map:
+			del cls._index_map[gen]
+	
+	def __typecheck__(self, gen, to_check):
+		index_map = self.__class__._index_map
+		
+		# There might be multiple YieldSeq's per signature
+		if self not in index_map[gen]:
+			index_map[gen][self] = -1
+		index = index_map[gen]
+
+		if index[self] >= len(self._types)-1:
+			raise _TC_YieldCountError(len(self._types))
+			
+		index[self] += 1		
+		check_type(self._types[index[self]], gen, to_check)
+				
+register_type(YieldSeq)
 
 # The current implementation of Self relies on the TypeVariables machinery
 def Self():
@@ -864,7 +1057,10 @@ def _make_fake_function(func):
 
 		return fake_function.__check_result(wrapped_func, result)
 	
-	def _pass_args(*vargs, **kwargs): pass
+	# These are the default implementations of __check_args
+	# and __check_results
+	def _pass_args(*vargs, **kwargs):
+		pass
 	def _pass_result(func, result):
 		stop_checking(func)
 		return result
@@ -879,10 +1075,6 @@ def _make_fake_function(func):
 	fake_function.__name__ = func.__name__
 	fake_function.__doc__ = func.__doc__
 	
-	for attr in ('args_pos', 'args_kw', 'args_kwargs', 'args_vargs', 'yield', 'return'):
-		attr = 'type_%s' % attr
-		if hasattr(func, attr):
-			setattr(fake_function, attr, getattr(func, attr))
 	return fake_function
 
 ###################################################
